#!/usr/bin/perl
#
# This is the subpixelgs evolver script which is distributed as part
# of the subpixelgs collection. It allows the user to choose
# a matrix for the subpixelize filter.
#
# Requirements: Perl/Tk and Imagemagick (which provides the display utility)
#
# Copyright (C) 2002 Laird Breyer
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Author:   Laird Breyer <laird@lbreyer.com>
#
use Tk;

$IDIR="$ENV{'HOME'}/bin";

time ^ $$;

# select the size of the grid
my $size = shift || 3;
$size = ($size > 2) ? $size : 3;

# work out the size of the sample image
my $buf = `head -4 ${IDIR}/small3.ppm`;
my ($w,$h) = ($buf =~ /\n(\d+) +(\d+)/s);
$w /= 3;

# set up a dummy matrix
my @default_matrix = ( 1, 1, 1, 0.5, 0.2, 0.1, 0, 0, 0,
		       0.1, 0.2, 0.5, 1, 1, 1, 0.5, 0.2, 0.1,
		       0, 0, 0, 0.1, 0.2, 0.5, 1, 1, 1);
# my @default_matrix = ( 1, 1, 1, 1, 1, 1, 1, 1, 1,
# 		       1, 1, 1, 1, 1, 1, 1, 1, 1,
# 		       1, 1, 1, 1, 1, 1, 1, 1, 1);

# this is the authoritative matrix
my @matrix = @default_matrix;
my $mat = "@default_matrix"; # textual description of @matrix

# see if user has a matrix file already
$usermat = `if [ -e \$HOME/.subpixelgs_rc ]; then source \$HOME/.subpixelgs_rc ; echo -ne "\$METHOD" ; fi`;
if( split(/ /, $usermat) == 28 ) {
  ($junk, @matrix) = split / /, $usermat;
  $mat = "@matrix";
}

# build the window
my $mw = MainWindow->new();
$mw->Tk::bind($mw,'<Enter>', \&initialize);

my $qbutton = $mw->Button(text => 'Quit', command => sub { exit; });
$qbutton->grid(-row =>($size+1), -column =>0);

my $rbutton = $mw->Button(text => 'Randomize current', command => \&randomize_current);
$rbutton->grid(-row =>($size+1), -column =>1);

my $sbutton = $mw->Button(text => 'Save current', command => \&save_current);
$sbutton->grid(-row =>($size+1), -column =>2);

my $matbox_label = $mw->Label(-text => 'current matrix:');
$matbox_label->grid(-row => 0, -row => ($size+2));
my $matbox = $mw->Entry(-width => 50, -textvariable => \$mat);
$matbox->grid(-row =>($size+2), -column => 1, -columnspan => $size);

my @windows;
my @matrices;
for($i = 0; $i < $size; $i++) {
  for($j = 0; $j < $size; $j++) {
    $matrices[$i][$j] = \@matrix;
    $window[$i][$j] = $mw->Canvas(-width => $w, -height => $h)->grid(-row => $i, -column => $j, -padx => 2, -pady => 2);
    $window[$i][$j]->Tk::bind($window[$i][$j],'<1>', [\&select_canvas, $i, $j]);
    $window[$i][$j]->Tk::bind($window[$i][$j],'<2>', [\&blank_canvas, $i, $j]);
  }
}

my $mframe = $mw->Frame()->grid(-row => ($size + 3), -column => 0, -columnspan => $size, -padx =>3, -pady =>3);
my $lleftw = $mframe->Label(-text => 'simple subpixel')->grid(-row => 0, -column => 0);
my $leftw = $mframe->Canvas(-width => $w, -height => $h)->grid(-row => 1, -column => 0, -padx => 2);

my $lmidw = $mframe->Label(-text => 'current selection')->grid(-row => 0, -column => 1);
my $midw = $mframe->Canvas(-width => $w, -height => $h)->grid(-row => 1, -column => 1, -padx => 2);

my $lrightw = $mframe->Label(-text => 'simple antialias')->grid(-row => 0, -column => 2);
my $rightw = $mframe->Canvas(-width => $w, -height => $h)->grid(-row => 1, -column => 2, -padx => 2);


MainLoop();


sub initialize {
$id = $leftw->id();
system("cat ${IDIR}/small3.ppm | subpixelize spr display -window $id -immutable ");

$id = $midw->id();
system("cat ${IDIR}/small3.ppm | subpixelize mat $mat display -window $id -immutable ");

$id = $rightw->id();
system("cat ${IDIR}/small3.ppm | subpixelize aa display -window $id -immutable ");

$mw->Tk::bind($mw,'<Enter>', ''); # only initialize once
}



sub randomize_current {
  if( split(/ /, $mat) == 27) {
    @matrix = split / /, $mat;
  } else {
    @matrix = @default_matrix;
    $mat = "@default_matrix";
  }
  for($i = 0; $i < $size; $i++) {
    for($j = 0; $j < $size; $j++) {
      my $wid = $window[$i][$j]->id();
      my @params = &randomize_matrix(@matrix);
      $matrices[$i][$j] = \@params;
      system("cat ${IDIR}/small3.ppm | subpixelize mat @params display -window $wid -immutable ");
    }
  }
}

sub save_current {
  if( split(/ /, $mat) == 27) {
    @matrix = split / /, $mat;
  } else {
    @matrix = @default_matrix;
    $mat = "@default_matrix";
  }
  system("echo -ne 'METHOD=\"mat @matrix\"\n' > \$HOME/.subpixelgs_rc");
}

sub select_canvas {
  my ($canvas, $i, $j) = @_;
  my $r = $matrices[$i][$j];
  @matrix = @$r;
  $mat = "@matrix";

  my $id = $midw->id();
  system("cat ${IDIR}/small3.ppm | subpixelize mat $mat display -window $id -immutable ");
}

sub blank_canvas {
  my ($canvas, $i, $j) = @_;
  my $wid = $canvas->id();
  my $r = $matrices[$i][$j];
  my @params = @$r;
  system("cat ${IDIR}/small3.ppm | subpixelize mat @params display -window $wid -negate ");
}

sub randomize_matrix {
  my @mat = @_;
  for($l = 0; $l < 27; $l++) {
    if(rand > 0.1) {
      my $newval = $mat[$l] + int(rand 4)/2 - 1;
      $mat[$l] = ($newval > 0) ? $newval : 0;
    }
  }
  return @mat;
}
