#!/bin/sh

INSTALLDIR=$HOME/bin

# check for available programs
if [ ! -e `which awk` ]; then
    echo "Could not find awk. Installation aborted. Please install awk first"
    exit 1
fi

if [ ! -e `which xprop` ]; then
    echo "Could not find xprop. Installation aborted. Please install xprop first"
    exit 1
fi

if [ ! -e `which xwininfo` ]; then
    echo "Could not find xwininfo. Installation aborted. Please install xwininfo first"
    exit 1
fi

if [ ! -e `which display` ] && [ ! -e `which xli` ] && [ ! -e `which xloadimage` ]; then
    echo "Could not find a valid image viewer. Installation aborted. Please install either Imagemagick or xloadimage first"
    exit 1
fi

# create/copy to install dir
[ -d $INSTALLDIR ] || mkdir $INSTALLDIR
cp ./subpixelgs.sh ./subpixelize ./evolver.pl ./small3.ppm $INSTALLDIR

# override ghostview settings
if [ -f $HOME/.gv ] && [ ! -f $HOME/.gv_backup ]; then
    mv $HOME/.gv $HOME/.gv_backup
    echo "Old gv settings have been saved in $HOME/.gv_backup"
fi
cat > $HOME/.gv <<EOF

GV.gsQuiet:             True
GV.gsSafer:             True
GV.gsArguments:         
GV.gsX11AlphaDevice:    -dNOPLATFONTS -sDEVICE=x11alpha
GV.gsX11Device:         -sDEVICE=x11
GV.gsCmdConvPDF:        pdf2ps %pdf %ps
GV.gsCmdScanPDF:        pdf2dsc %pdf %dsc
GV.gsInterpreter:       subpixelgs.sh

GV.scale:               1
GV.scaleBase:           1
GV.fallbackOrientation: Portrait
GV.orientation:         Automatic
GV.fallbackPageMedia:   A4
GV.pageMedia:           Automatic
GV.swapLandscape:       False
GV.autoResize:          True
GV.antialias:           False
GV.watchFile:           False
GV.ignoreEOF:           True
GV.respectDSC:          True
EOF

echo "Installation completed successfully."
echo "Programs were copied to $INSTALLDIR."
echo "You must make sure this directory is in your path, or the scripts"
echo "will not work."
echo ""
echo "If you use bash, edit your file $HOME/.bashrc and append the line:"
echo "export PATH=\$PATH:$INSTALLDIR"
echo ""
echo "If you use tcsh, edit your file $HOME/.cshrc and append the line:"
echo "setenv PATH \$PATH:$INSTALLDIR"