#!/bin/sh
# This is a wrapper around gs which redirects
# the x11 output to allow subpixel rendering.
# If the parent process is not gv, or the output
# device is not x11, this wrapper simply runs gs.
#
#
# Copyright (C) 2002 Laird Breyer
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Author:   Laird Breyer <laird@lbreyer.com>
#

# where the supbixelize filter program resides
INSTPATH=${HOME}/bin

# wrapper only works if spawned by gv
if [ -z ${GHOSTVIEW} ]; then
    exec gs $@;
fi

# wrapper only works if DEVICE=x11
PARAMETERS=`echo $* | sed -e 's/x11 /ppmraw /'`
if [ "$*" == "${PARAMETERS}" ]; then
    exec gs $@;
fi

# check for awk
if [ ! -e `which awk` ]; then 
    echo "$0 error: this script needs awk"
    exit
fi

# check for a suitable viewing program
if [ -e `which xli` ]; then
    HEXID=`printf "%x" ${GHOSTVIEW}`
    VIEWER="xli -windowid 0x${HEXID} stdin 2>&1 > /dev/null"
elif [ -e `which display` ]; then
    VIEWER="display -window ${GHOSTVIEW} -immutable"
else
    echo "$0 error: no suitable image viewer"
    exit
fi

# since we're redirecting the output, 
# we don't want a conflict with an existing output file
PARAMETERS=`echo ${PARAMETERS} | sed -e 's/-sOutputFile=[^ ]*//'`

# now we read the output window's size and display resolution
WIDTH=`xwininfo -id ${GHOSTVIEW} | awk '/Width: / { print 3 * $2; }'`
HEIGHT=`xwininfo -id ${GHOSTVIEW} | awk '/Height: / { print $2; }'`

XDPI=`xprop -id ${GHOSTVIEW} | awk '/GHOSTVIEW\(STRING\)/ { print 3 * int($9); }'`
YDPI=`xprop -id ${GHOSTVIEW} | awk '/GHOSTVIEW\(STRING\)/ { print int($10); }'`

# here we set the default rendering method
METHOD="spr2"

# configuration file override
[ -e /etc/subpixelgs_rc ] && source /etc/subpixelgs_rc
# this overrides METHOD
[ -e ${HOME}/.subpixelgs_rc ] && source ${HOME}/.subpixelgs_rc

# call ghostscript to produce an image 3 times wider than required
# output is piped to subpixelize, which slims down the image by a factor of 3
gs -r${XDPI}x${YDPI} -g${WIDTH}x${HEIGHT} -sOutputFile=- ${PARAMETERS} | ${INSTPATH}/subpixelize ${METHOD} ${VIEWER}
